#ifndef UMLWIDGET_H
#define UMLWIDGET_H
#include <qobject.h>
#include <qstring.h>
#include <qcanvas.h>

#include "umlnamespace.h"
#include "umlobject.h"
#include "listpopupmenu.h"
#include "umlwidgetdata.h"
#include "umlview.h"

class CAssociationWidget;
class QPainter;

/**
*	This is the base class for nearly all graphical widgets.
*
*	@short	The base class for graphical UML objects.
*	@author Paul Hensgen <phensgen@techie.com>
*	@version 1.0
*/


class UMLWidget :  public QObject, public QCanvasRectangle, public Uml
{
Q_OBJECT

friend class UMLView;

public:
  /**
  * Creates a UMLWidget object.	
  *	
  *	@param	view		The view to be displayed on.
  *	@param	o				The UMLObject to represent.
  *  @param pData     The CUMLWidgetData for this UMLWidget, it must not be null
  */
	UMLWidget( UMLView * view, UMLObject * o, CUMLWidgetData * pData );
	
	/**
		*	Creates a UMLWidget object.		
		*	
		*	@param	view		The view to be displayed on.
		*	@param	id				The id of the widget.
		*  @param pData     The CUMLWidgetData for this UMLWidget, it must not be null
		*/
	UMLWidget( UMLView * view, int id,  CUMLWidgetData * pData);
	
	/**
		*	Creates a UMLWidget object.	
		*	
		*	@param	view		The view to be displayed on.
		*  @param pData     The CUMLWidgetData for this UMLWidget, it must not be null	
		*/
	UMLWidget( UMLView * view,  CUMLWidgetData * pData );
	
	/**
		*	Standard deconstructor	
		*/
	~UMLWidget();
	
	/**
		*	Returns the type of object this represents.
		*
		*	@return	Returns the type of object this represents.
		*/
	virtual Uml::UMLWidget_Type getBaseType();
	
	/**
		*	Returns the ID of this object.
		*
		*	@return	Returns the ID this object.
		*/
	virtual int getID();
	
	/**
		* Sets the id og the UMLWidget.
		*
		*  @param id The id to set the widget to.
		*/
	virtual void setID(int Id){m_pData->m_nId = Id;}
	
	/**
		*	Adds one to the amount of association this object is connected to.	
		*/
	void addAssoc(CAssociationWidget* pAssoc){m_pData->AddAssoc(pAssoc);}
	
	/**
		*	Removes one from the count of the amount of associations this object
		*	is connected to.	
		*/
	void removeAssoc(CAssociationWidget* pAssoc){m_pData->RemoveAssoc(pAssoc);}
	
	/**
		*	Returns the @ref UMLObject set to represent.
		*/
	UMLObject * getUMLObject(){return m_pObject;}
	
	/**
		*	Sets the @ref UMLObject to represent.
		*
		*	@param	o	The object to represent.	
		*/
	void setUMLObject(UMLObject * o){m_pObject = o;}
	
	/**
 		* Use to save or load this classes information
  	*
  	*	@param	s	Pointer to the datastream (file) to save/load from.
  	*	@param	archive	If true will save the classes information, else will
  	* load the information.
  	*
  	*	@return	Returns the status of the operation.
  	*/
	virtual bool serialize(QDataStream *s, bool archive);

  /**
  * Returns the amount of bytes needed to serialize an instance object to the clipboard
  */
  virtual long getClipSizeOf();

	/**
		*	Overrides the standard operation.
		*/
  virtual void mouseReleaseEvent(QMouseEvent * me);

  /**
    *		Overrides the standard operation.
    */
  virtual void mouseDoubleClickEvent(QMouseEvent * /*me*/);

  /**
		*	Return the status of using fill color.
		*
		*	@return	Return the status of using fill color.
		*/
  bool getUseFillColor(){return m_pData->m_bUseFillColor;}

  /**
		*	Return the background color.
		*
		*	@return	Return the background color.
		*/
  QColor getBGColor(){return m_FillColor;}

  /**
		*	Return the line color.
		*
		*	@return Return the line color.
		*/
  QColor getLineColor(){return m_LineColor;}

  /**
  	 *		Overrides the standard operation.

  	 */
  virtual void mouseMoveEvent(QMouseEvent *me);

  /**
     *	Returns wether this is a line of text.
     *	Used for transparency in printing.
     *
     */
  virtual bool isText(){return false;}

  /**
		*	Set the status of using fill color.
		*
		*	@param	fc the status of using fill color.

		*/

  void setUseFillColor(bool fc);

  /**
		*	Sets the state of whether the widget is selected.
		*
		*	@param _select The state of whether the widget is selected.
		*/
  virtual void setSelected(bool _select);

  /**
		*	Returns the state of whether the widget is selected.
		*
		*	@return Returns the state of whether the widget is selected.
		*/
  bool getSelected(){return m_bSelected;}

  /**
		*	Sets the view the widget is on.
		*
		*	@param v The view the widget is on.
		*/
  void setView(UMLView * v);

  /**
		*	Returns the old id of the widget.  Used for copy/cut operations.
		*
		*	@return Returns the old id of the widget.
		*/
  virtual int getOldID(){return m_nOldID;}

  /**
		*	Sets the old id of the widget.  Used for copy/cut operations.
		*
		*	@param _id Sets the old id of the widget.
		*/
  virtual void setOldID(int _id){m_nOldID = _id;}

  /**
  * Activate the object after serializing it from a QDataStream
  */
  virtual bool Activate(CIDChangeLog* ChangeLog = 0);

  /**
  * Returns true if the given point is in the boundaries of the widget
  */
  virtual bool onWidget(const QPoint & p);

  /**
  * Draws the UMLWidget on the given paint device
  */
  virtual void draw( QPainter & p, int offsetX, int offsetY );

	/**
	*  	Sets the font the widget is to use.
	*/

	virtual void setFont( QFont font ){ m_pData -> setFont( font ); calculateSize(); update(); }

	/**
	* 		Returns the font the widget is too use.
	*/
	virtual QFont getFont(){ return m_pData -> getFont(); }

	/**
	*		Returns whether we triggered the update of position movement.
	*		If so, you probably don't want to move it.
	*/
	bool getStartMove(){ return m_bStartMove; }

  /**
  *   Sets the x-coordinate.
  */
  void setX( int x );

  /**
  *   Sets the y-coordinate.
  */
  void setY( int y );

  /**
  *   Used by some child classes to get documentation.
  */
  virtual QString getDoc(){ return ""; }

  /**
  *   Used by some child classes to set documentation.
  */
  virtual void setDoc( QString ){};

  /**
  * Returns true if the Activate method has been called for this instance
  */
  virtual const bool& IsActivated();

  /**
  * Return a CUMLWidgetData Containing all information about this instance of UMLWidget
  */
  virtual CUMLWidgetData* getData();

  /**
  * Synchronize the Widget's m_pData member with its display properties, for example:
  * the X and Y positions of the widget, etc

  */
  virtual void SynchronizeData();

  /**
  * Sets the m_Name property
  */
  virtual void SetName(QString strName);

  /**
  * Returns a copy of m_Name
  */
  virtual QString GetName(){ return m_Name; }

  /**
  * Starts the popup menu.
  */
  void StartPopupMenu(QPoint At);

  /**
  *   Adjusts associations with the given co-ordinates
   */
  void AdjustAssocs(int x, int y);
  /**
  * Set the m_bActivated flag of a widget but does not perform the Activate method
  */
  void SetActivated(bool Active = true);

  /**
  *   Used to cleanup any other widget it may need to delete.
  *   Used by child classes.  This should be called before deleting a widget of a diagram.
  */
  virtual void cleanup();

protected:
  /**

  *  Draws that the widhet is selected.
  */
  virtual void drawSelected(QPainter * p, int offsetX, int offsetY);

  /**
  *   Overrides default method.

  */
  virtual void drawShape(QPainter &p );

  /**
  *	Overrides the standard operation.
  */
  virtual void mousePressEvent(QMouseEvent *me);

   /**
   	*	Overrides the standard operation.
   	*/
	virtual void moveEvent(QMoveEvent */*m*/){};
	
	/**
		*	Calculates the size of the widget.

		*/
	virtual void calculateSize(){}
	
	/**
		*	Initializes key attributes of the class.
		*/
  void init();

  bool m_bMouseDown, m_bMouseOver, m_bSelected, m_bStartMove;
  int  m_nOldX, m_nOldY, m_nPosX, m_nOldID;
  UMLObject *m_pObject;
  UMLView * m_pView;
  ListPopupMenu * m_pMenu;	
  QColor m_FillColor, m_LineColor;
	bool m_bResizing;
  CUMLWidgetData* m_pData;
  int m_nPressOffsetX, m_nPressOffsetY;
	int m_nOldH, m_nOldW;

  /**
  * It is true if the Activate Function has been called for this

  * class instance
  */
  bool m_bActivated;
  /**
  * UMLWidget's name property
  */
  QString m_Name;

public slots:

  /**
  * This slot is entered when an event has occurred on the views display,
  * most likely a mouse event.  Before it sends out that mouse event all
  * children should make sure that they don't have a menu active or there
  * could be more than one popup menu displayed.
  */
  virtual void slotRemovePopupMenu();

   /**
  *   Whena widget changes this slot captures that signal.
   */
  virtual void slotChangeWidget(UMLObject * o);

   /**
  *   Captures any popup menu signals for menus it created.
   */
  virtual void slotMenuSelection(int sel);

   /**
  *   Captures when another widget moves if it is link to it that signal.
   */
  virtual void slotWidgetMoved(int id);

   /**
  *     Captures a color change signal.
   */
  virtual void slotColorChanged(int viewID);

  /**
  *   Captures a sigClearAllSelected signal sent by @ref UMLView
  */
  void slotClearAllSelected();

	/**
	* 		Tells the widget to snap to grid.
	*		Will use the grid settings of the @ref UMLView it belongs to.
	*/
	void slotSnapToGrid();

signals:
  /**
  *   Emit when the widget moves its' position.
  */
	void sigWidgetMoved(int);
};

#endif
